@@ -9,6 +9,7 @@ use {
     },
     rand_chacha::ChaChaRng,
     std::{
+        borrow::Borrow,
         iter,
         ops::{AddAssign, Div, Sub, SubAssign},
     },
@@ -134,20 +135,51 @@ where
     }
 }
 
+// Equivalent to WeightedShuffle(rng, weights).unwrap().next().
+pub fn weighted_sample_single<R: Rng, T>(rng: &mut R, cumulative_weights: &[T]) -> Option<usize>
+where
+    T: Copy + Default + PartialOrd + SampleUniform,
+{
+    let zero = <T as Default>::default();
+    let high = cumulative_weights.last().copied().unwrap_or_default();
+    #[allow(clippy::neg_cmp_op_on_partial_ord)]
+    if !(high > zero) {
+        return None;
+    }
+    let sample = <T as SampleUniform>::Sampler::sample_single(zero, high, rng);
+    let mut lo = 0usize;
+    let mut hi = cumulative_weights.len() - 1;
+    while lo + 1 < hi {
+        let k = lo + (hi - lo) / 2;
+        if cumulative_weights[k] <= sample {
+            lo = k;
+        } else {
+            hi = k;
+        }
+    }
+    if cumulative_weights[lo] > sample {
+        Some(lo)
+    } else {
+        Some(hi)
+    }
+}
+
 /// Returns a list of indexes shuffled based on the input weights
 /// Note - The sum of all weights must not exceed `u64::MAX`
-pub fn weighted_shuffle<T>(weights: &[T], seed: [u8; 32]) -> Vec<usize>
+pub fn weighted_shuffle<T, B, F>(weights: F, seed: [u8; 32]) -> Vec<usize>
 where
     T: Copy + PartialOrd + iter::Sum + Div<T, Output = T> + FromPrimitive + ToPrimitive,
+    B: Borrow<T>,
+    F: Iterator<Item = B> + Clone,
 {
-    let total_weight: T = weights.iter().copied().sum();
+    let total_weight: T = weights.clone().map(|x| *x.borrow()).sum();
     let mut rng = ChaChaRng::from_seed(seed);
     weights
-        .iter()
         .enumerate()
-        .map(|(i, v)| {
+        .map(|(i, weight)| {
+            let weight = weight.borrow();
             // This generates an "inverse" weight but it avoids floating point math
-            let x = (total_weight / *v)
+            let x = (total_weight / *weight)
                 .to_u64()
                 .expect("values > u64::max are not supported");
             (
@@ -225,7 +257,7 @@ mod tests {
     fn test_weighted_shuffle_iterator() {
         let mut test_set = [0; 6];
         let mut count = 0;
-        let shuffle = weighted_shuffle(&[50, 10, 2, 1, 1, 1], [0x5a; 32]);
+        let shuffle = weighted_shuffle(vec![50, 10, 2, 1, 1, 1].into_iter(), [0x5a; 32]);
         shuffle.into_iter().for_each(|x| {
             assert_eq!(test_set[x], 0);
             test_set[x] = 1;
@@ -240,7 +272,7 @@ mod tests {
         let mut test_weights = vec![0; 100];
         (0..100).for_each(|i| test_weights[i] = (i + 1) as u64);
         let mut count = 0;
-        let shuffle = weighted_shuffle(&test_weights, [0xa5; 32]);
+        let shuffle = weighted_shuffle(test_weights.into_iter(), [0xa5; 32]);
         shuffle.into_iter().for_each(|x| {
             assert_eq!(test_set[x], 0);
             test_set[x] = 1;
@@ -251,9 +283,9 @@ mod tests {
 
     #[test]
     fn test_weighted_shuffle_compare() {
-        let shuffle = weighted_shuffle(&[50, 10, 2, 1, 1, 1], [0x5a; 32]);
+        let shuffle = weighted_shuffle(vec![50, 10, 2, 1, 1, 1].into_iter(), [0x5a; 32]);
 
-        let shuffle1 = weighted_shuffle(&[50, 10, 2, 1, 1, 1], [0x5a; 32]);
+        let shuffle1 = weighted_shuffle(vec![50, 10, 2, 1, 1, 1].into_iter(), [0x5a; 32]);
         shuffle1
             .into_iter()
             .zip(shuffle.into_iter())
@@ -266,7 +298,7 @@ mod tests {
     fn test_weighted_shuffle_imbalanced() {
         let mut weights = vec![std::u32::MAX as u64; 3];
         weights.push(1);
-        let shuffle = weighted_shuffle(&weights, [0x5a; 32]);
+        let shuffle = weighted_shuffle(weights.iter().cloned(), [0x5a; 32]);
         shuffle.into_iter().for_each(|x| {
             if x == weights.len() - 1 {
                 assert_eq!(weights[x], 1);
@@ -287,20 +319,40 @@ mod tests {
         assert_eq!(best_index, 2);
     }
 
+    // Asserts that empty weights will return empty shuffle.
+    #[test]
+    fn test_weighted_shuffle_empty_weights() {
+        let weights = Vec::<u64>::new();
+        let mut rng = rand::thread_rng();
+        let shuffle = WeightedShuffle::new(&mut rng, &weights);
+        assert!(shuffle.unwrap().next().is_none());
+        assert_eq!(weighted_sample_single(&mut rng, &weights), None);
+    }
+
+    // Asserts that zero weights will return empty shuffle.
+    #[test]
+    fn test_weighted_shuffle_zero_weights() {
+        let weights = vec![0u64; 5];
+        let mut rng = rand::thread_rng();
+        let shuffle = WeightedShuffle::new(&mut rng, &weights);
+        assert!(shuffle.unwrap().next().is_none());
+        assert_eq!(weighted_sample_single(&mut rng, &weights), None);
+    }
+
     // Asserts that each index is selected proportional to its weight.
     #[test]
     fn test_weighted_shuffle_sanity() {
         let seed: Vec<_> = (1..).step_by(3).take(32).collect();
         let seed: [u8; 32] = seed.try_into().unwrap();
         let mut rng = ChaChaRng::from_seed(seed);
-        let weights = [1, 1000, 10, 100];
-        let mut counts = [0; 4];
+        let weights = [1, 0, 1000, 0, 0, 10, 100, 0];
+        let mut counts = [0; 8];
         for _ in 0..100000 {
             let mut shuffle = WeightedShuffle::new(&mut rng, &weights).unwrap();
             counts[shuffle.next().unwrap()] += 1;
             let _ = shuffle.count(); // consume the rest.
         }
-        assert_eq!(counts, [101, 90113, 891, 8895]);
+        assert_eq!(counts, [101, 0, 90113, 0, 0, 891, 8895, 0]);
     }
 
     #[test]
@@ -308,6 +360,13 @@ mod tests {
         let weights = [
             78, 70, 38, 27, 21, 0, 82, 42, 21, 77, 77, 17, 4, 50, 96, 83, 33, 16, 72,
         ];
+        let cumulative_weights: Vec<_> = weights
+            .iter()
+            .scan(0, |acc, w| {
+                *acc += w;
+                Some(*acc)
+            })
+            .collect();
         let seed = [48u8; 32];
         let mut rng = ChaChaRng::from_seed(seed);
         let shuffle: Vec<_> = WeightedShuffle::new(&mut rng, &weights).unwrap().collect();
@@ -315,6 +374,11 @@ mod tests {
             shuffle,
             [2, 11, 16, 0, 13, 14, 15, 10, 1, 9, 7, 6, 12, 18, 4, 17, 3, 8]
         );
+        let mut rng = ChaChaRng::from_seed(seed);
+        assert_eq!(
+            weighted_sample_single(&mut rng, &cumulative_weights),
+            Some(2),
+        );
         let seed = [37u8; 32];
         let mut rng = ChaChaRng::from_seed(seed);
         let shuffle: Vec<_> = WeightedShuffle::new(&mut rng, &weights).unwrap().collect();
@@ -322,12 +386,24 @@ mod tests {
             shuffle,
             [17, 3, 14, 13, 6, 10, 15, 16, 9, 2, 4, 1, 0, 7, 8, 18, 11, 12]
         );
+        let mut rng = ChaChaRng::from_seed(seed);
+        assert_eq!(
+            weighted_sample_single(&mut rng, &cumulative_weights),
+            Some(17),
+        );
     }
 
     #[test]
     fn test_weighted_shuffle_match_slow() {
         let mut rng = rand::thread_rng();
         let weights: Vec<u64> = repeat_with(|| rng.gen_range(0, 1000)).take(997).collect();
+        let cumulative_weights: Vec<_> = weights
+            .iter()
+            .scan(0, |acc, w| {
+                *acc += w;
+                Some(*acc)
+            })
+            .collect();
         for _ in 0..10 {
             let mut seed = [0u8; 32];
             rng.fill(&mut seed[..]);
@@ -335,7 +411,12 @@ mod tests {
             let shuffle: Vec<_> = WeightedShuffle::new(&mut rng, &weights).unwrap().collect();
             let mut rng = ChaChaRng::from_seed(seed);
             let shuffle_slow = weighted_shuffle_slow(&mut rng, weights.clone());
-            assert_eq!(shuffle, shuffle_slow,);
+            assert_eq!(shuffle, shuffle_slow);
+            let mut rng = ChaChaRng::from_seed(seed);
+            assert_eq!(
+                weighted_sample_single(&mut rng, &cumulative_weights),
+                Some(shuffle[0]),
+            );
         }
     }
 }
